@@ -112,15 +112,6 @@ class TrainCache(Generic[ModelT]):
                     return is_config_consistent(path)
 
         base_path = cls.train_result_base_path(project_path)
-        if not base_path.exists():
-            base_path = project_path / "train_result"
-
-            yellow = "\x1b[33;20m"
-            warn_msg = yellow
-            warn_msg += "please rename path/to/project/train_result to path/to/project/models."
-            warn_msg += "\n Otherwise, your model could not be loaded in the future release"
-            warnings.warn(warn_msg, DeprecationWarning)
-
         ps = filter(filter_predicate, base_path.iterdir())
         return list(ps)
 
@@ -206,7 +197,7 @@ class TrainCache(Generic[ModelT]):
         assert m is not None
         file_uuid = m[3]
 
-        best_model: ModelT = torch.load(model_path)
+        best_model: ModelT = torch.load(model_path, map_location=torch.device("cpu"))
         best_model.put_on_device(torch.device("cpu"))
         train_loss = cls.load_lossseq_table_from_npz_dict(np.load(train_loss_path))
         valid_loss = cls.load_lossseq_table_from_npz_dict(np.load(valid_loss_path))
@@ -223,7 +214,14 @@ class TrainCache(Generic[ModelT]):
         ps = cls.filter_result_paths(project_path, model_type, **kwargs)
         tcache_list = [cls.load_from_base_path(p) for p in ps]
         if len(tcache_list) == 0:
-            raise FileNotFoundError
+            if model_type is None:
+                model_name = "<not-specified>"
+            else:
+                model_name = model_type.__name__
+            message = "[query] model_name => {}, kwargs => {}, project_path => {}".format(
+                model_name, kwargs, project_path
+            )
+            raise FileNotFoundError(message)
 
         return tcache_list
 
